SystemOrganization addCategory: #DijEvolution!

Object subclass: #DijEvolution
	instanceVariableNames: 'ancestors leastCostLife breedingCode childrenOfLeastCostLife costSortedLives'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution'!

!DijEvolution classSide methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 11:11'!
forGraph

	^self new
		 breedingCode: 
			[:parent| |v nei|
			v:= parent object.
			nei:= v neighbors.
			nei collect: [:n|  |e|
				e:= (v edgesConnectingTo: n) anyOne.  
				parent giveBirthWith: (Message selector:#otherEndPointVia: argument: e) cost: e cost  ].
			].! !

!DijEvolution classSide methodsFor: 'examples' stamp: 'hjo 6/28/2012 11:14'!
modulo17
|evo sol|
evo := DijEvolution new.
evo addSeed: 1.
evo breedingCode: [:parent| #(2 3) collect: [:m| parent giveBirthWith:(Message selector:#multiply:modulo: arguments: (Array with: m with: 17)) ] ].  
evo step.
evo runUntilEqual: 7.

sol := evo ancestors last.

sol birthMessageChain.
sol birthMessagesReplayed.
sol object.
sol cost.  ! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 11:59'!
addChildrenToCostSortedLives

	costSortedLives addAll: childrenOfLeastCostLife.! !

!DijEvolution methodsFor: 'adding' stamp: 'hjo 6/28/2012 11:59'!
addSeed: anObject

	costSortedLives add: (DijLife object: anObject cost: 0).! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:49'!
ancestors

	^ ancestors! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 6/28/2012 00:38'!
breedingCode: aBlock
	"[:parent| #(2 3) collect: [:m| parent giveBirthWith:(Message selector:#multiply:modulo: arguments: (Array with: m with: 17)) ] ]"
	breedingCode := aBlock
! !

!DijEvolution methodsFor: 'testing' stamp: 'hjo 6/28/2012 11:59'!
canStep

	^costSortedLives notEmpty! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 6/28/2012 11:59'!
costSortedLives

	^ costSortedLives! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 01:21'!
dropChildren

	| ancestorObjects |
	ancestorObjects := ancestors collect: [:each| each object ]. 
	childrenOfLeastCostLife := childrenOfLeastCostLife reject: [:ch| ancestorObjects includes: ch object ]! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 11:59'!
getLeastCostLifeFromSorted
	leastCostLife := costSortedLives removeFirst.! !

!DijEvolution methodsFor: 'initialize-release' stamp: 'hjo 6/28/2012 12:05'!
initialize

	super initialize.
	ancestors := OrderedCollection new.
	costSortedLives := SortedCollection sortBlock: (UnarySortDescriptor ascendingBy: #cost).! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/27/2012 17:00'!
jionLeastCostLifeToAncestors
	ancestors addLast: leastCostLife.! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 12:02'!
leaveOnlyTheLeastCostLifeIfThereAreManyLivesOfSameObjectsInTheCostSortedObjects

	| survival |
	survival := LeastCostSurvivalOfSameObject new.
	survival evolution: self.
	survival clearLivesToRemoveFromSorted.
	survival getAssociationsOfLivesOfEqualObjectFromCostSortedLivesOfEvolution.
	survival collectLivesToRemoveFromSorted.
	survival removeLivesToRemoveFromSortedFromCostSortedLivesOfEvolution.! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 01:21'!
makeChildren
	childrenOfLeastCostLife := breedingCode value: leastCostLife.! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 6/28/2012 09:01'!
run

	[self canStep ifTrue: [self step. true] ifFalse: [false]
	] whileTrue.! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 6/28/2012 00:42'!
runUntilEqual: anObject

	| findObject |
	findObject := [ ancestors anySatisfy: [:l | l object = anObject] ].
	[self canStep and: [self step. findObject value not ]
	] whileTrue.
	^findObject value! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 6/28/2012 11:17'!
step

	self getLeastCostLifeFromSorted.
	self jionLeastCostLifeToAncestors.
	self makeChildren.
	self dropChildren.
	self addChildrenToCostSortedLives.
	self leaveOnlyTheLeastCostLifeIfThereAreManyLivesOfSameObjectsInTheCostSortedObjects.
! !

Object subclass: #DijLife
	instanceVariableNames: 'origin birthMessage birthCost cost object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution'!

!DijLife classSide methodsFor: 'instance creation' stamp: 'hjo 6/27/2012 14:18'!
object: anObject 

	^self new object: anObject! !

!DijLife classSide methodsFor: 'instance creation' stamp: 'hjo 6/27/2012 14:18'!
object: anObject cost: aNumber

	^self new object: anObject; cost: aNumber! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
birthCost
	^ birthCost! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/28/2012 00:53'!
birthCost: aNumber
	birthCost := aNumber! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
birthMessage
	^ birthMessage! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/28/2012 00:53'!
birthMessage: aMessage
	birthMessage := aMessage! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 21:58'!
birthMessageChain

	^Array streamContents: [:stream| self birthMessageChainOn: stream]! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 21:58'!
birthMessageChainOn: stream

	origin notNil
		ifTrue: [ origin birthMessageChainOn: stream. stream nextPut: birthMessage ].! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 21:52'!
birthMessagesReplayed

	origin isNil ifTrue: [^object].
	^birthMessage sendTo: origin birthMessagesReplayed. ! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 14:59'!
computeCost

	cost := origin cost + birthCost.! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 04:08'!
cost
	^ cost! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 04:08'!
cost: aNumber
	cost := aNumber
! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 23:14'!
giveBirthWith: aMessage 

	^self giveBirthWith: aMessage cost: 1! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 23:13'!
giveBirthWith: aMessage cost: aNumber

	^self species new 
		origin: self;
		birthMessage: aMessage;
		birthCost: aNumber;
		object: (aMessage sendTo: object);
		computeCost  ! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 14:50'!
object

	^ object! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:57'!
object: anObject 

	object := anObject! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
origin
	^ origin! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
origin: anObject
	origin := anObject! !

!DijLife methodsFor: 'printing' stamp: 'hjo 6/27/2012 18:03'!
printOn: aStream

	aStream nextPut: $[.
	aStream nextPutAll: 'o:'.
	aStream print: object.
	aStream space.
	aStream nextPutAll: 'c:'.
	aStream print: cost.
	aStream nextPut: $].! !

Object subclass: #LeastCostSurvivalOfSameObject
	instanceVariableNames: 'livesToRemoveFromSorted evolution associationsOfLifesOfEqualObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution'!

!LeastCostSurvivalOfSameObject methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 01:38'!
clearLivesToRemoveFromSorted
	livesToRemoveFromSorted := OrderedCollection new.! !

!LeastCostSurvivalOfSameObject methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 12:06'!
collectLivesToRemoveFromSorted
	associationsOfLifesOfEqualObjects do: [:ass| "object -> lives"
		| leastCostObject  |
		leastCostObject := (ass value asSortedCollection: (UnarySortDescriptor ascendingBy: #cost)) first.  
		livesToRemoveFromSorted addAll: (ass value copyWithout: leastCostObject)].
! !

!LeastCostSurvivalOfSameObject methodsFor: 'accessing' stamp: 'hjo 6/28/2012 01:41'!
evolution: aDijEvolution 

	evolution:= aDijEvolution ! !

!LeastCostSurvivalOfSameObject methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 14:34'!
getAssociationsOfLivesOfEqualObjectFromCostSortedLivesOfEvolution
	associationsOfLifesOfEqualObjects:= ((evolution costSortedLives groupBy: #object) associations select: [:each| each value size > 1 ]).! !

!LeastCostSurvivalOfSameObject methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 14:34'!
removeLivesToRemoveFromSortedFromCostSortedLivesOfEvolution
	evolution costSortedLives removeAll: livesToRemoveFromSorted.! !

!Integer methodsFor: '*DijEvolution' stamp: 'hjo 6/27/2012 15:03'!
multiply: aSmallInteger modulo: aSmallInteger2 

	^(self * aSmallInteger) \\ aSmallInteger2! !
